==> de-unkrig-commons-1.1.1-sources.jar: de/unkrig/commons/lang/ThreadUtil.java <==

/*
 * de.unkrig.commons - A general-purpose Java class library
 *
 * Copyright (c) 2011, Arno Unkrig
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *       following disclaimer.
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *       following disclaimer in the documentation and/or other materials provided with the distribution.
 *    3. The name of the author may not be used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package de.unkrig.commons.lang;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.unkrig.commons.lang.protocol.HardReference;
import de.unkrig.commons.lang.protocol.RunnableWhichThrows;
import de.unkrig.commons.lang.protocol.Stoppable;
import de.unkrig.commons.nullanalysis.Nullable;

/**
 * Various {@code java.lang.Thread}-related utility methods.
 */
public final
class ThreadUtil {

    private static final Logger LOGGER = Logger.getLogger(ThreadUtil.class.getName());

    private
    ThreadUtil() {}

    /**
     * Execute the given {@code runnable} in a background thread
     *
     * @return The beckground runnable can be interrupted through this object
     */
    public static Stoppable
    runInBackground(Runnable runnable, @Nullable String threadName) {
        final Thread t = new Thread(runnable, threadName == null ? runnable.toString() : threadName);
        t.setDaemon(true);

        if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) ThreadUtil.LOGGER.log(
            Level.FINE,
            "Starting daemon thread {0} / ''{1}''",
            new Object[] { t.getId(), t.getName() }
        );

        t.start();

        return new Stoppable() {

            @Override public void
            stop() {
                if (!t.isAlive()) {
                    if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) {
                        ThreadUtil.LOGGER.fine("Background runnable '" + t.getName() + "' is not running");
                    }
                    return;
                }

                if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) {
                    ThreadUtil.LOGGER.fine("Stopping background runnable '" + t.getName() + "'");
                }

                t.interrupt();
                try {
                    t.join();
                } catch (InterruptedException ie) {
                    return;
                }

                if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) {
                    ThreadUtil.LOGGER.fine("Background runnable '" + t.getName() + "' stopped");
                }
            }
        };
    }

    /**
     * @see #runInBackground(Runnable, String)
     */
    public static <EX extends Exception> Stoppable
    runInBackground(final RunnableWhichThrows<EX> runnable, @Nullable String threadName) {
        return ThreadUtil.runInBackground(new Runnable() {

            @Override public void
            run() {
                try {
                    runnable.run();
                } catch (Exception e) {
                    ThreadUtil.LOGGER.log(Level.INFO, "Exception caught from runnable", e);
                }
            }
        }, threadName);
    }

    /**
     * Executes the given {@code runnable}.
     */
    public static <EX extends Exception> void
    runInForeground(RunnableWhichThrows<EX> runnable) throws EX {
        runnable.run();
    }

    /**
     * Executes the two {@code runnable}s in parallel before it returns.
     */
    public static <EX extends Exception> void
    parallel(
        RunnableWhichThrows<EX> runnable1,
        RunnableWhichThrows<EX> runnable2,
        Stoppable               stoppable
    ) throws EX {
        List<RunnableWhichThrows<EX>> l = new ArrayList<RunnableWhichThrows<EX>>();
        l.add(runnable1);
        l.add(runnable2);
        ThreadUtil.parallel(l, Collections.<Stoppable>singleton(stoppable));
    }

    /**
     * @see #parallel(Runnable[], Iterable)
     */
    public static <EX extends Exception> void
    parallel(Iterable<? extends RunnableWhichThrows<EX>> runnables) throws EX {
        ThreadUtil.parallel(runnables, Collections.<Stoppable>emptyList());
    }

    /**
     * @see #parallel(Runnable[], Iterable)
     */
    public static <EX extends Exception> void
    parallel(Iterable<? extends RunnableWhichThrows<EX>> runnables, Iterable<Stoppable> stoppables) throws EX {

        final HardReference<EX> caughtException = new HardReference<EX>();

        List<Runnable> l = new ArrayList<Runnable>();
        for (final RunnableWhichThrows<EX> runnable : runnables) {
            l.add(new Runnable() {

                @Override public void
                run() {
                    try {
                        runnable.run();
                    } catch (RuntimeException re) {
                        throw re;
                    } catch (Exception e) {

                        // "e" can only be an EX, because that is the only checked exception that the delegate
                        // runnables will throw.
                        @SuppressWarnings("unchecked") EX ex = (EX) e;
                        caughtException.set(ex);
                    }
                }
            });
        }

        ThreadUtil.parallel(l.toArray(new Runnable[l.size()]), stoppables);

        EX ex = caughtException.get();
        if (ex != null) throw ex;
    }

    /**
     * Invokes the {@link Runnable#run run} method of all {@code runnables} in parallel threads (including the current
     * thread). When the first of these invocations returns, then all threads are {@link Thread#interrupt()
     * interrupt}ed (which awakes them from blocking I/O), and also on all {@code stoppables} {@link Stoppable#stop()}
     * is called. When all the threads have been {@link Thread#join() join}ed, this method returns.
     */
    public static void
    parallel(Runnable[] runnables, final Iterable<Stoppable> stoppables) {

        final HardReference<RuntimeException> caughtRuntimeException = new HardReference<RuntimeException>();

        // Run all runnables.
        final Thread[] threads = new Thread[runnables.length];
        for (int i = 0; i < runnables.length; i++) {
            final Runnable runnable  = runnables[i];
            Runnable       runnable2 = new Runnable() {

                @Override public void
                run() {
                    try {
                        runnable.run();
                    } catch (RuntimeException re) {
                        caughtRuntimeException.set(re);
                    }
                    for (Thread thread : threads) {
                        if (thread != null) thread.interrupt();
                    }
                    for (Stoppable stoppable : stoppables) {
                        if (stoppable != null) stoppable.stop();
                    }
                }
            };

            Thread t;
            if (i < runnables.length - 1) {
                t = new Thread(runnable2);
                t.setName(Thread.currentThread().getName());
                t.start();
            } else {
                runnable2.run();
                t = Thread.currentThread();
            }
            threads[i] = t;
        }

        // Join pending threads.
        for (Thread thread : threads) {
            for (;;) {
                try {
                    thread.join();
                    break;
                } catch (InterruptedException e) {
                    ;
                }
            }
        }

        RuntimeException re = caughtRuntimeException.get();
        if (re != null) throw re;
    }

    /**
     * Runs all but the last of {@code runnables} in the background, and the last of {@code runnables} in the
     * foreground.
     */
    public static <R extends RunnableWhichThrows<EX>, EX extends Exception> void
    runInForeground(Iterable<R> runnables) throws EX {
        Iterator<R>     it         = runnables.iterator();
        List<Stoppable> stoppables = new ArrayList<Stoppable>();
        for (;;) {
            RunnableWhichThrows<EX> r = it.next();
            if (!it.hasNext()) {
                ThreadUtil.runInForeground(r);
                break;
            }
            stoppables.add(ThreadUtil.runInBackground(r, null));
        }
        for (Stoppable stoppable : stoppables) stoppable.stop();
    }
}

==> de-unkrig-commons-1.1.4-sources.jar: de/unkrig/commons/lang/ThreadUtil.java <==

/*
 * de.unkrig.commons - A general-purpose Java class library
 *
 * Copyright (c) 2011, Arno Unkrig
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *       following disclaimer.
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *       following disclaimer in the documentation and/or other materials provided with the distribution.
 *    3. The name of the author may not be used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package de.unkrig.commons.lang;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.unkrig.commons.lang.protocol.HardReference;
import de.unkrig.commons.lang.protocol.RunnableWhichThrows;
import de.unkrig.commons.lang.protocol.Stoppable;
import de.unkrig.commons.nullanalysis.Nullable;

/**
 * Various {@code java.lang.Thread}-related utility methods.
 */
public final
class ThreadUtil {

    private static final Logger LOGGER = Logger.getLogger(ThreadUtil.class.getName());

    private
    ThreadUtil() {}

    /**
     * Execute the given {@code runnable} in a background thread
     *
     * @return The beckground runnable can be interrupted through this object
     */
    public static Stoppable
    runInBackground(Runnable runnable, @Nullable String threadName) {
        final Thread t = new Thread(runnable, threadName == null ? runnable.toString() : threadName);
        t.setDaemon(true);

        if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) ThreadUtil.LOGGER.log(
            Level.FINE,
            "Starting daemon thread {0} / ''{1}''",
            new Object[] { t.getId(), t.getName() }
        );

        t.start();

        return new Stoppable() {

            @Override public void
            stop() {
                if (!t.isAlive()) {
                    if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) {
                        ThreadUtil.LOGGER.fine("Background runnable '" + t.getName() + "' is not running");
                    }
                    return;
                }

                if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) {
                    ThreadUtil.LOGGER.fine("Stopping background runnable '" + t.getName() + "'");
                }

                t.interrupt();
                try {
                    t.join();
                } catch (InterruptedException ie) {
                    return;
                }

                if (ThreadUtil.LOGGER.isLoggable(Level.FINE)) {
                    ThreadUtil.LOGGER.fine("Background runnable '" + t.getName() + "' stopped");
                }
            }
        };
    }

    /**
     * @see #runInBackground(Runnable, String)
     */
    public static <EX extends Exception> Stoppable
    runInBackground(final RunnableWhichThrows<EX> runnable, @Nullable String threadName) {
        return ThreadUtil.runInBackground(new Runnable() {

            @Override public void
            run() {
                try {
                    runnable.run();
                } catch (Exception e) {
                    ThreadUtil.LOGGER.log(Level.INFO, "Exception caught from runnable", e);
                }
            }
        }, threadName);
    }

    /**
     * Executes the given {@code runnable}.
     */
    public static <EX extends Exception> void
    runInForeground(RunnableWhichThrows<EX> runnable) throws EX {
        runnable.run();
    }

    /**
     * Executes the two {@code runnable}s in parallel before it returns.
     */
    public static <EX extends Exception> void
    parallel(
        RunnableWhichThrows<EX> runnable1,
        RunnableWhichThrows<EX> runnable2,
        Stoppable               stoppable
    ) throws EX {
        List<RunnableWhichThrows<EX>> l = new ArrayList<RunnableWhichThrows<EX>>();
        l.add(runnable1);
        l.add(runnable2);
        ThreadUtil.parallel(l, Collections.<Stoppable>singleton(stoppable));
    }

    /**
     * @see #parallel(Runnable[], Iterable)
     */
    public static <EX extends Exception> void
    parallel(Iterable<? extends RunnableWhichThrows<EX>> runnables) throws EX {
        ThreadUtil.parallel(runnables, Collections.<Stoppable>emptyList());
    }

    /**
     * @see #parallel(Runnable[], Iterable)
     */
    public static <EX extends Exception> void
    parallel(Iterable<? extends RunnableWhichThrows<EX>> runnables, Iterable<Stoppable> stoppables) throws EX {

        final HardReference<EX> caughtException = new HardReference<EX>();

        List<Runnable> l = new ArrayList<Runnable>();
        for (final RunnableWhichThrows<EX> runnable : runnables) {
            l.add(new Runnable() {

                @Override public void
                run() {
                    try {
                        runnable.run();
                    } catch (RuntimeException re) {
                        throw re;
                    } catch (Exception e) {

                        // "e" can only be an EX, because that is the only checked exception that the delegate
                        // runnables will throw.
                        @SuppressWarnings("unchecked") EX ex = (EX) e;
                        caughtException.set(ex);
                    }
                }
            });
        }

        ThreadUtil.parallel(l.toArray(new Runnable[l.size()]), stoppables);

        EX ex = caughtException.get();
        if (ex != null) throw ex;
    }

    /**
     * Invokes the {@link Runnable#run run} method of all {@code runnables} in parallel threads (including the current
     * thread). When the first of these invocations returns, then all threads are {@link Thread#interrupt()
     * interrupt}ed (which awakes them from blocking I/O), and also on all {@code stoppables} {@link Stoppable#stop()}
     * is called. When all the threads have been {@link Thread#join() join}ed, this method returns.
     */
    public static void
    parallel(Runnable[] runnables, final Iterable<Stoppable> stoppables) {

        final HardReference<RuntimeException> caughtRuntimeException = new HardReference<RuntimeException>();

        // Run all runnables.
        final Thread[] threads = new Thread[runnables.length - 1];
        for (int i = 0; i < runnables.length; i++) {
            final Runnable runnable  = runnables[i];
            Runnable       runnable2 = new Runnable() {

                @Override public void
                run() {
                    try {
                        runnable.run();
                    } catch (RuntimeException re) {
                        caughtRuntimeException.set(re);
                    }
                    for (Thread thread : threads) {
                        if (thread != null) thread.interrupt();
                    }
                    for (Stoppable stoppable : stoppables) {
                        if (stoppable != null) stoppable.stop();
                    }
                }
            };

            Thread t;
            if (i < runnables.length - 1) {
                t = new Thread(runnable2);
                t.setName(Thread.currentThread().getName());
                t.start();
                threads[i] = t;
            } else {
                runnable2.run();
            }
        }

        // Join pending threads.
        for (Thread thread : threads) {
            for (;;) {
                try {
                    thread.join();
                    break;
                } catch (InterruptedException e) {
                    ;
                }
            }
        }

        RuntimeException re = caughtRuntimeException.get();
        if (re != null) throw re;
    }

    /**
     * Runs all but the last of {@code runnables} in the background, and the last of {@code runnables} in the
     * foreground.
     */
    public static <R extends RunnableWhichThrows<EX>, EX extends Exception> void
    runInForeground(Iterable<R> runnables) throws EX {
        Iterator<R>     it         = runnables.iterator();
        List<Stoppable> stoppables = new ArrayList<Stoppable>();
        for (;;) {
            RunnableWhichThrows<EX> r = it.next();
            if (!it.hasNext()) {
                ThreadUtil.runInForeground(r);
                break;
            }
            stoppables.add(ThreadUtil.runInBackground(r, null));
        }
        for (Stoppable stoppable : stoppables) stoppable.stop();
    }
}
